@@ -23,6 +23,7 @@ from pandas._typing import (
     DtypeObj,
     Manager,
 )
+from pandas.util._exceptions import find_stack_level
 
 from pandas.core.dtypes.cast import (
     construct_1d_arraylike_from_scalar,
@@ -65,11 +66,11 @@ from pandas.core.construction import (
     range_to_ndarray,
     sanitize_array,
 )
-from pandas.core.indexes import base as ibase
 from pandas.core.indexes.api import (
     DatetimeIndex,
     Index,
     TimedeltaIndex,
+    default_index,
     ensure_index,
     get_objs_combined_axis,
     union_indexes,
@@ -79,14 +80,15 @@ from pandas.core.internals.array_manager import (
     SingleArrayManager,
 )
 from pandas.core.internals.blocks import (
+    BlockPlacement,
     ensure_block_shape,
-    new_block,
+    new_block_2d,
 )
 from pandas.core.internals.managers import (
     BlockManager,
     SingleBlockManager,
-    create_block_manager_from_arrays,
     create_block_manager_from_blocks,
+    create_block_manager_from_column_arrays,
 )
 
 if TYPE_CHECKING:
@@ -99,9 +101,8 @@ if TYPE_CHECKING:
 
 def arrays_to_mgr(
     arrays,
-    arr_names: Index,
+    columns: Index,
     index,
-    columns,
     *,
     dtype: DtypeObj | None = None,
     verify_integrity: bool = True,
@@ -122,23 +123,40 @@ def arrays_to_mgr(
 
         # don't force copy because getting jammed in an ndarray anyway
         arrays = _homogenize(arrays, index, dtype)
+        # _homogenize ensures
+        #  - all(len(x) == len(index) for x in arrays)
+        #  - all(x.ndim == 1 for x in arrays)
+        #  - all(isinstance(x, (np.ndarray, ExtensionArray)) for x in arrays)
+        #  - all(type(x) is not PandasArray for x in arrays)
 
     else:
         index = ensure_index(index)
+        arrays = [extract_array(x, extract_numpy=True) for x in arrays]
+
+        # Reached via DataFrame._from_arrays; we do validation here
+        for arr in arrays:
+            if (
+                not isinstance(arr, (np.ndarray, ExtensionArray))
+                or arr.ndim != 1
+                or len(arr) != len(index)
+            ):
+                raise ValueError(
+                    "Arrays must be 1-dimensional np.ndarray or ExtensionArray "
+                    "with length matching len(index)"
+                )
 
     columns = ensure_index(columns)
+    if len(columns) != len(arrays):
+        raise ValueError("len(arrays) must match len(columns)")
 
     # from BlockManager perspective
     axes = [columns, index]
 
     if typ == "block":
-        return create_block_manager_from_arrays(
-            arrays, arr_names, axes, consolidate=consolidate
+        return create_block_manager_from_column_arrays(
+            arrays, axes, consolidate=consolidate
         )
     elif typ == "array":
-        if len(columns) != len(arrays):
-            assert len(arrays) == 0
-            arrays = [np.array([], dtype=object) for _ in range(len(columns))]
         return ArrayManager(arrays, [index, columns])
     else:
         raise ValueError(f"'typ' needs to be one of {{'block', 'array'}}, got '{typ}'")
@@ -158,7 +176,7 @@ def rec_array_to_mgr(
     # essentially process a record array then fill it
     fdata = ma.getdata(data)
     if index is None:
-        index = _get_names_from_index(fdata)
+        index = default_index(len(fdata))
     else:
         index = ensure_index(index)
 
@@ -180,14 +198,14 @@ def rec_array_to_mgr(
     # create the manager
 
     # error: Argument 1 to "reorder_arrays" has incompatible type "List[ndarray]";
-    # expected "List[ExtensionArray]"
+    # expected "List[Union[ExtensionArray, ndarray]]"
     arrays, arr_columns = reorder_arrays(
-        new_arrays, arr_columns, columns  # type: ignore[arg-type]
+        new_arrays, arr_columns, columns, len(index)  # type: ignore[arg-type]
     )
     if columns is None:
         columns = arr_columns
 
-    mgr = arrays_to_mgr(arrays, arr_columns, index, columns, dtype=dtype, typ=typ)
+    mgr = arrays_to_mgr(arrays, columns, index, dtype=dtype, typ=typ)
 
     if copy:
         mgr = mgr.copy()
@@ -226,7 +244,7 @@ def mgr_to_mgr(mgr, typ: str, copy: bool = True):
         else:
             if mgr.ndim == 2:
                 new_mgr = arrays_to_mgr(
-                    mgr.arrays, mgr.axes[0], mgr.axes[1], mgr.axes[0], typ="block"
+                    mgr.arrays, mgr.axes[0], mgr.axes[1], typ="block"
                 )
             else:
                 new_mgr = SingleBlockManager.from_array(mgr.arrays[0], mgr.index)
@@ -272,14 +290,23 @@ def ndarray_to_mgr(
         if not len(values) and columns is not None and len(columns):
             values = np.empty((0, 1), dtype=object)
 
+    # if the array preparation does a copy -> avoid this for ArrayManager,
+    # since the copy is done on conversion to 1D arrays
+    copy_on_sanitize = False if typ == "array" else copy
+
     vdtype = getattr(values, "dtype", None)
-    if is_1d_only_ea_dtype(vdtype) or isinstance(dtype, ExtensionDtype):
+    if is_1d_only_ea_dtype(vdtype) or is_1d_only_ea_dtype(dtype):
         # GH#19157
 
-        if isinstance(values, np.ndarray) and values.ndim > 1:
+        if isinstance(values, (np.ndarray, ExtensionArray)) and values.ndim > 1:
             # GH#12513 a EA dtype passed with a 2D array, split into
             #  multiple EAs that view the values
-            values = [values[:, n] for n in range(values.shape[1])]
+            # error: No overload variant of "__getitem__" of "ExtensionArray"
+            # matches argument type "Tuple[slice, int]"
+            values = [
+                values[:, n]  # type: ignore[call-overload]
+                for n in range(values.shape[1])
+            ]
         else:
             values = [values]
 
@@ -288,10 +315,10 @@ def ndarray_to_mgr(
         else:
             columns = ensure_index(columns)
 
-        return arrays_to_mgr(values, columns, index, columns, dtype=dtype, typ=typ)
+        return arrays_to_mgr(values, columns, index, dtype=dtype, typ=typ)
 
     elif is_extension_array_dtype(vdtype) and not is_1d_only_ea_dtype(vdtype):
-        # i.e. Datetime64TZ
+        # i.e. Datetime64TZ, PeriodDtype
         values = extract_array(values, extract_numpy=True)
         if copy:
             values = values.copy()
@@ -301,7 +328,7 @@ def ndarray_to_mgr(
     else:
         # by definition an array here
         # the dtypes will be coerced to a single dtype
-        values = _prep_ndarray(values, copy=copy)
+        values = _prep_ndarray(values, copy=copy_on_sanitize)
 
     if dtype is not None and not is_dtype_equal(values.dtype, dtype):
         shape = values.shape
@@ -311,7 +338,7 @@ def ndarray_to_mgr(
         rcf = not (is_integer_dtype(dtype) and values.dtype.kind == "f")
 
         values = sanitize_array(
-            flat, None, dtype=dtype, copy=copy, raise_cast_failure=rcf
+            flat, None, dtype=dtype, copy=copy_on_sanitize, raise_cast_failure=rcf
         )
 
         values = values.reshape(shape)
@@ -331,14 +358,17 @@ def ndarray_to_mgr(
         if dtype is None and is_object_dtype(values.dtype):
             arrays = [
                 ensure_wrapped_if_datetimelike(
-                    maybe_infer_to_datetimelike(values[:, i].copy())
+                    maybe_infer_to_datetimelike(values[:, i])
                 )
                 for i in range(values.shape[1])
             ]
         else:
             if is_datetime_or_timedelta_dtype(values.dtype):
                 values = ensure_wrapped_if_datetimelike(values)
-            arrays = [values[:, i].copy() for i in range(values.shape[1])]
+            arrays = [values[:, i] for i in range(values.shape[1])]
+
+        if copy:
+            arrays = [arr.copy() for arr in arrays]
 
         return ArrayManager(arrays, [index, columns], verify_integrity=False)
 
@@ -348,31 +378,30 @@ def ndarray_to_mgr(
     # on the entire block; this is to convert if we have datetimelike's
     # embedded in an object type
     if dtype is None and is_object_dtype(values.dtype):
-
-        if values.ndim == 2 and values.shape[0] != 1:
-            # transpose and separate blocks
-
-            dtlike_vals = [maybe_infer_to_datetimelike(row) for row in values]
-            dvals_list = [ensure_block_shape(dval, 2) for dval in dtlike_vals]
-
-            # TODO: What about re-joining object columns?
+        obj_columns = list(values)
+        maybe_datetime = [maybe_infer_to_datetimelike(x) for x in obj_columns]
+        # don't convert (and copy) the objects if no type inference occurs
+        if any(x is not y for x, y in zip(obj_columns, maybe_datetime)):
+            dvals_list = [ensure_block_shape(dval, 2) for dval in maybe_datetime]
             block_values = [
-                new_block(dvals_list[n], placement=n, ndim=2)
+                new_block_2d(dvals_list[n], placement=BlockPlacement(n))
                 for n in range(len(dvals_list))
             ]
-
         else:
-            datelike_vals = maybe_infer_to_datetimelike(values)
-            nb = new_block(datelike_vals, placement=slice(len(columns)), ndim=2)
+            bp = BlockPlacement(slice(len(columns)))
+            nb = new_block_2d(values, placement=bp)
             block_values = [nb]
     else:
-        nb = new_block(values, placement=slice(len(columns)), ndim=2)
+        bp = BlockPlacement(slice(len(columns)))
+        nb = new_block_2d(values, placement=bp)
         block_values = [nb]
 
     if len(columns) == 0:
         block_values = []
 
-    return create_block_manager_from_blocks(block_values, [columns, index])
+    return create_block_manager_from_blocks(
+        block_values, [columns, index], verify_integrity=False
+    )
 
 
 def _check_values_indices_shape_match(
@@ -414,7 +443,6 @@ def dict_to_mgr(
         from pandas.core.series import Series
 
         arrays = Series(data, index=columns, dtype=object)
-        data_names = arrays.index
         missing = arrays.isna()
         if index is None:
             # GH10856
@@ -427,21 +455,31 @@ def dict_to_mgr(
         if missing.any() and not is_integer_dtype(dtype):
             nan_dtype: DtypeObj
 
-            if dtype is None or (
-                isinstance(dtype, np.dtype) and np.issubdtype(dtype, np.flexible)
-            ):
+            if dtype is not None:
+                # calling sanitize_array ensures we don't mix-and-match
+                #  NA dtypes
+                midxs = missing.values.nonzero()[0]
+                for i in midxs:
+                    arr = sanitize_array(arrays.iat[i], index, dtype=dtype)
+                    arrays.iat[i] = arr
+            else:
                 # GH#1783
                 nan_dtype = np.dtype("object")
-            else:
-                nan_dtype = dtype
-            val = construct_1d_arraylike_from_scalar(np.nan, len(index), nan_dtype)
-            arrays.loc[missing] = [val] * missing.sum()
+                val = construct_1d_arraylike_from_scalar(np.nan, len(index), nan_dtype)
+                nmissing = missing.sum()
+                if copy:
+                    rhs = [val] * nmissing
+                else:
+                    # GH#45369
+                    rhs = [val.copy() for _ in range(nmissing)]
+                arrays.loc[missing] = rhs
 
         arrays = list(arrays)
+        columns = ensure_index(columns)
 
     else:
         keys = list(data.keys())
-        columns = data_names = Index(keys)
+        columns = Index(keys)
         arrays = [com.maybe_iterable_to_list(data[k]) for k in keys]
         # GH#24096 need copy to be deep for datetime64tz case
         # TODO: See if we can avoid these copies
@@ -461,9 +499,7 @@ def dict_to_mgr(
         ]
         # TODO: can we get rid of the dt64tz special case above?
 
-    return arrays_to_mgr(
-        arrays, data_names, index, columns, dtype=dtype, typ=typ, consolidate=copy
-    )
+    return arrays_to_mgr(arrays, columns, index, dtype=dtype, typ=typ, consolidate=copy)
 
 
 def nested_data_to_arrays(
@@ -488,9 +524,9 @@ def nested_data_to_arrays(
             index = _get_names_from_index(data)
         elif isinstance(data[0], Categorical):
             # GH#38845 hit in test_constructor_categorical
-            index = ibase.default_index(len(data[0]))
+            index = default_index(len(data[0]))
         else:
-            index = ibase.default_index(len(data))
+            index = default_index(len(data))
 
     return arrays, columns, index
 
@@ -589,6 +625,7 @@ def _homogenize(data, index: Index, dtype: DtypeObj | None) -> list[ArrayLike]:
             val = sanitize_array(
                 val, index, dtype=dtype, copy=False, raise_cast_failure=False
             )
+            com.require_length_match(val, index)
 
         homogenized.append(val)
 
@@ -602,7 +639,7 @@ def _extract_index(data) -> Index:
     index = None
     if len(data) == 0:
         index = Index([])
-    elif len(data) > 0:
+    else:
         raw_lengths = []
         indexes: list[list[Hashable] | Index] = []
 
@@ -620,11 +657,13 @@ def _extract_index(data) -> Index:
             elif is_list_like(val) and getattr(val, "ndim", 1) == 1:
                 have_raw_arrays = True
                 raw_lengths.append(len(val))
+            elif isinstance(val, np.ndarray) and val.ndim > 1:
+                raise ValueError("Per-column arrays must each be 1-dimensional")
 
         if not indexes and not raw_lengths:
             raise ValueError("If using all scalar values, you must pass an index")
 
-        if have_series:
+        elif have_series:
             index = union_indexes(indexes)
         elif have_dicts:
             index = union_indexes(indexes, sort=False)
@@ -648,7 +687,7 @@ def _extract_index(data) -> Index:
                     )
                     raise ValueError(msg)
             else:
-                index = ibase.default_index(lengths[0])
+                index = default_index(lengths[0])
 
     # error: Argument 1 to "ensure_index" has incompatible type "Optional[Index]";
     # expected "Union[Union[Union[ExtensionArray, ndarray], Index, Series],
@@ -657,20 +696,40 @@ def _extract_index(data) -> Index:
 
 
 def reorder_arrays(
-    arrays: list[ArrayLike], arr_columns: Index, columns: Index | None
+    arrays: list[ArrayLike], arr_columns: Index, columns: Index | None, length: int
 ) -> tuple[list[ArrayLike], Index]:
+    """
+    Pre-emptively (cheaply) reindex arrays with new columns.
+    """
     # reorder according to the columns
-    if columns is not None and len(columns) and len(arr_columns):
-        indexer = ensure_index(arr_columns).get_indexer(columns)
-        arr_columns = ensure_index([arr_columns[i] for i in indexer])
-        arrays = [arrays[i] for i in indexer]
+    if columns is not None:
+        if not columns.equals(arr_columns):
+            # if they are equal, there is nothing to do
+            new_arrays: list[ArrayLike | None]
+            new_arrays = [None] * len(columns)
+            indexer = arr_columns.get_indexer(columns)
+            for i, k in enumerate(indexer):
+                if k == -1:
+                    # by convention default is all-NaN object dtype
+                    arr = np.empty(length, dtype=object)
+                    arr.fill(np.nan)
+                else:
+                    arr = arrays[k]
+                new_arrays[i] = arr
+
+            # Incompatible types in assignment (expression has type
+            # "List[Union[ExtensionArray, ndarray[Any, Any], None]]", variable
+            # has type "List[Union[ExtensionArray, ndarray[Any, Any]]]")
+            arrays = new_arrays  # type: ignore[assignment]
+            arr_columns = columns
+
     return arrays, arr_columns
 
 
 def _get_names_from_index(data) -> Index:
     has_some_name = any(getattr(s, "name", None) is not None for s in data)
     if not has_some_name:
-        return ibase.default_index(len(data))
+        return default_index(len(data))
 
     index: list[Hashable] = list(range(len(data)))
     count = 0
@@ -692,12 +751,12 @@ def _get_axes(
     # return axes or defaults
 
     if index is None:
-        index = ibase.default_index(N)
+        index = default_index(N)
     else:
         index = ensure_index(index)
 
     if columns is None:
-        columns = ibase.default_index(K)
+        columns = default_index(K)
     else:
         columns = ensure_index(columns)
     return index, columns
@@ -717,13 +776,14 @@ def dataclasses_to_dicts(data):
 
     Examples
     --------
+    >>> from dataclasses import dataclass
     >>> @dataclass
-    >>> class Point:
+    ... class Point:
     ...     x: int
     ...     y: int
 
-    >>> dataclasses_to_dicts([Point(1,2), Point(2,3)])
-    [{"x":1,"y":2},{"x":2,"y":3}]
+    >>> dataclasses_to_dicts([Point(1, 2), Point(2, 3)])
+    [{'x': 1, 'y': 2}, {'x': 2, 'y': 3}]
 
     """
     from dataclasses import asdict
@@ -740,6 +800,17 @@ def to_arrays(
 ) -> tuple[list[ArrayLike], Index]:
     """
     Return list of arrays, columns.
+
+    Returns
+    -------
+    list[ArrayLike]
+        These will become columns in a DataFrame.
+    Index
+        This will become frame.columns.
+
+    Notes
+    -----
+    Ensures that len(result_arrays) == len(result_index).
     """
     if isinstance(data, ABCDataFrame):
         # see test_from_records_with_index_data, test_from_records_bad_index_column
@@ -781,10 +852,15 @@ def to_arrays(
             "To retain the old behavior, pass as a dictionary "
             "DataFrame({col: categorical, ..})",
             FutureWarning,
-            stacklevel=4,
+            stacklevel=find_stack_level(),
         )
         if columns is None:
-            columns = ibase.default_index(len(data))
+            columns = default_index(len(data))
+        elif len(columns) > len(data):
+            raise ValueError("len(columns) > len(data)")
+        elif len(columns) < len(data):
+            # doing this here is akin to a pre-emptive reindex
+            data = data[: len(columns)]
         return data, columns
 
     elif isinstance(data, np.ndarray) and data.dtype.names is not None:
@@ -836,7 +912,7 @@ def _list_of_series_to_arrays(
     for s in data:
         index = getattr(s, "index", None)
         if index is None:
-            index = ibase.default_index(len(s))
+            index = default_index(len(s))
 
         if id(index) in indexer_cache:
             indexer = indexer_cache[id(index)]
@@ -941,7 +1017,7 @@ def _validate_or_indexify_columns(
         not equal to length of content
     """
     if columns is None:
-        columns = ibase.default_index(len(content))
+        columns = default_index(len(content))
     else:
 
         # Add mask for data which is composed of list of lists
