@@ -2,9 +2,13 @@ import numpy as np
 import pytest
 
 import pandas as pd
-from pandas import Index, MultiIndex
+from pandas import (
+    Index,
+    MultiIndex,
+)
 
 
+# Note: identical the the "multi" entry in the top-level "index" fixture
 @pytest.fixture
 def idx():
     # a MultiIndex used to test the general functionality of the
@@ -49,18 +53,6 @@ def index_names():
     return ["first", "second"]
 
 
-@pytest.fixture
-def holder():
-    # the MultiIndex constructor used to base compatibility with pickle
-    return MultiIndex
-
-
-@pytest.fixture
-def compat_props():
-    # a MultiIndex must have these properties associated with it
-    return ["shape", "ndim", "size"]
-
-
 @pytest.fixture
 def narrow_multi_index():
     """
@@ -69,7 +61,7 @@ def narrow_multi_index():
     n = 1000
     ci = pd.CategoricalIndex(list("a" * n) + (["abc"] * n))
     dti = pd.date_range("2000-01-01", freq="s", periods=n * 2)
-    return pd.MultiIndex.from_arrays([ci, ci.codes + 9, dti], names=["a", "b", "dti"])
+    return MultiIndex.from_arrays([ci, ci.codes + 9, dti], names=["a", "b", "dti"])
 
 
 @pytest.fixture
@@ -82,4 +74,4 @@ def wide_multi_index():
     dti = pd.date_range("2000-01-01", freq="s", periods=n * 2)
     levels = [ci, ci.codes + 9, dti, dti, dti]
     names = ["a", "b", "dti_1", "dti_2", "dti_3"]
-    return pd.MultiIndex.from_arrays(levels, names=names)
+    return MultiIndex.from_arrays(levels, names=names)
